@@ -1,6 +1,7 @@
 /*
  *   SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
  *   SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ *   SPDX-FileCopyrightText: 2024 Harald Sitter <sitter@kde.org>
  *
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
@@ -1082,19 +1083,6 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
     }
 
     AppStream::Pool *pool = new AppStream::Pool;
-    auto fw = new QFutureWatcher<bool>(this);
-    connect(fw, &QFutureWatcher<bool>::finished, this, [this, fw, pool, source]() {
-        source->m_pool = pool;
-        m_flatpakLoadingSources.removeAll(source);
-        if (fw->result()) {
-            m_flatpakSources += source;
-        } else {
-            qWarning() << "Could not open the AppStream metadata pool" << pool->lastError();
-        }
-        metadataRefreshed(source->remote());
-        acquireFetching(false);
-        fw->deleteLater();
-    });
     acquireFetching(true);
 
 #ifdef APPSTREAM_NEW_POOL_API
@@ -1115,9 +1103,32 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
     QDir().mkpath(pool->cacheLocation());
 #endif
 
-    fw->setFuture(QtConcurrent::run(&m_threadPool, [pool] {
-        return pool->load();
-    }));
+    const auto loadDone = [this, source, pool](bool result) {
+        source->m_pool = pool;
+        m_flatpakLoadingSources.removeAll(source);
+        if (result) {
+            m_flatpakSources += source;
+        } else {
+            qWarning() << "Could not open the AppStream metadata pool" << pool->lastError();
+        }
+        metadataRefreshed(source->remote());
+        acquireFetching(false);
+    };
+
+    // We do not want to block the GUI for very long, so we load appstream pools via queued
+    // invocation both for the actual load and then for the post-load state changes.
+    QMetaObject::invokeMethod(
+        this,
+        [this, pool, loadDone] {
+            auto result = pool->load();
+            QMetaObject::invokeMethod(
+                this,
+                [loadDone, result] {
+                    loadDone(result);
+                },
+                Qt::QueuedConnection);
+        },
+        Qt::QueuedConnection);
 }
 
 QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
@@ -1451,11 +1462,15 @@ bool FlatpakBackend::flatpakResourceLessThan(AbstractResource *l, AbstractResour
     // clang-format on
 }
 
-ResultsStream *FlatpakBackend::deferredResultStream(const QString &streamName, std::function<void(ResultsStream *)> callback)
+template<typename F>
+ResultsStream *FlatpakBackend::deferredResultStream(const QString &streamName, const F &callback)
 {
-    ResultsStream *stream = new ResultsStream(streamName);
+    QPointer stream{new ResultsStream(streamName)};
 
     auto f = [stream, callback = std::move(callback)] {
+        if (!stream) { // already has been consumed somewhere else
+            return;
+        }
         callback(stream);
     };
 
@@ -1481,9 +1496,10 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
     } else if (!filter.resourceUrl.isEmpty()) {
         return new ResultsStream(QStringLiteral("FlatpakStream-void"), {});
     } else if (filter.state == AbstractResource::Upgradeable) {
-        return deferredResultStream(u"FlatpakStream-upgradeable"_s, [this](ResultsStream *stream) {
+        return deferredResultStream(u"FlatpakStream-upgradeable"_s, [this](const QPointer<ResultsStream> &stream) {
             auto fw = new QFutureWatcher<QHash<FlatpakInstallation *, QVector<FlatpakInstalledRef *>>>(this);
-            connect(fw, &QFutureWatcher<QHash<FlatpakInstallation *, QVector<FlatpakInstalledRef *>>>::finished, this, [this, fw, stream]() {
+            connect(fw, &QFutureWatcher<QHash<FlatpakInstallation *, QVector<FlatpakInstalledRef *>>>::finished, stream, [this, fw, stream]() {
+                // `stream` doesn't outlive `this`, so when we get called both will be still alive!
                 if (g_cancellable_is_cancelled(m_cancellable)) {
                     stream->finish();
                     fw->deleteLater();
