@@ -310,11 +310,7 @@ void PackageKitBackend::reloadPackageList()
 
     m_appdata.reset(new AppStream::Pool);
 
-    auto fw = new QFutureWatcher<bool>(this);
-    connect(fw, &QFutureWatcher<bool>::finished, this, [this, fw]() {
-        const auto correct = fw->result();
-        fw->deleteLater();
-
+    const auto loadDone = [this](bool correct) {
         if (!correct && m_packages.packages.isEmpty()) {
             QTimer::singleShot(0, this, [this]() {
                 Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
@@ -359,9 +355,22 @@ void PackageKitBackend::reloadPackageList()
                 }
             }
         }
-    });
-    fw->setFuture(QtConcurrent::run(&m_threadPool, &loadAppStream, m_appdata.get()));
-    fw->future().waitForFinished();
+    };
+
+    // We do not want to block the GUI for very long, so we load appstream pools via queued
+    // invocation both for the actual load and then for the post-load state changes.
+    QMetaObject::invokeMethod(
+        this,
+        [this, loadDone] {
+            auto result = loadAppStream(m_appdata.get());
+            QMetaObject::invokeMethod(
+                this,
+                [loadDone, result] {
+                    loadDone(result);
+                },
+                Qt::QueuedConnection);
+        },
+        Qt::QueuedConnection);
 }
 
 AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Component &component) const
@@ -672,6 +681,7 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
             if (!stream) {
                 return;
             }
+            loadAllPackages();
             const auto toResolve = kFilter<QVector<AbstractResource *>>(m_packages.packages, needsResolveFilter);
 
             auto installedAndNameFilter = [filter](AbstractResource *res) {
@@ -719,6 +729,7 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
             if (!stream) {
                 return;
             }
+            loadAllPackages();
             auto resources = kFilter<QVector<AbstractResource *>>(m_packages.packages, [](AbstractResource *res) {
                 return res->type() != AbstractResource::Technical && !qobject_cast<PackageKitResource *>(res)->isCritical()
                     && !qobject_cast<PackageKitResource *>(res)->extendsItself();
@@ -1134,4 +1145,19 @@ InlineMessage *PackageKitBackend::explainDysfunction() const
     return AbstractResourcesBackend::explainDysfunction();
 }
 
+void PackageKitBackend::loadAllPackages()
+{
+    if (m_allPackagesLoaded) {
+        return;
+    }
+    const auto components = m_appdata->components();
+    for (const auto &component : components) {
+        if (!component.packageNames().isEmpty()) {
+            addComponent(component);
+        }
+    }
+    includePackagesToAdd();
+    m_allPackagesLoaded = true;
+}
+
 #include "PackageKitBackend.moc"
