@@ -18,7 +18,7 @@ class QSnapdClient;
 class SnapResource : public AbstractResource
 {
     Q_OBJECT
-    Q_PROPERTY(QStringList topObjects MEMBER m_objects CONSTANT)
+    Q_PROPERTY(QStringList topObjects MEMBER m_topObjects CONSTANT)
     Q_PROPERTY(QString channel READ channel WRITE setChannel NOTIFY channelChanged)
 public:
     explicit SnapResource(QSharedPointer<QSnapdSnap> snap, AbstractResource::State state, SnapBackend *parent);
